/*
 * AVL_Tree.icc    (c) Tommy Olsson, IDA, 2007-05-02
 */
#include <iostream>
#include <algorithm>
#include <stdexcept>
#include "AVL_Tree.h"
using namespace std;

/*
 * AVL_Tree_Node.
 */
template<typename Comparable>
class AVL_Tree_Node 
{
   friend class AVL_Tree<Comparable>;

   AVL_Tree_Node(const Comparable& element)
      : element_(element), left_(0), right_(0), height_(0) {}

   AVL_Tree_Node(const Comparable& element, AVL_Tree_Node* left, AVL_Tree_Node* right)
      : element_(element), left_(left), right_(right), height_(0) {}

   Comparable     element_;
   AVL_Tree_Node* left_;
   AVL_Tree_Node* right_;
   int            height_;

   typedef AVL_Tree_Node<Comparable>  Node;
   typedef AVL_Tree_Node<Comparable>* Node_Pointer;

   // Fljande funktioner anropas av motsvarande funktioner i AVL_Tree.
   static void insert(const Comparable&, Node_Pointer&);
   static void remove(const Comparable&, Node_Pointer&);
   static void clear(Node_Pointer&);

   static Node_Pointer find(const Comparable&, const Node_Pointer);
   static Node_Pointer find_min(const Node_Pointer);
   static Node_Pointer find_max(const Node_Pointer);

   static void print(std::ostream&, const Node_Pointer);
   static void print_tree(std::ostream&, const Node_Pointer, int depth = 0);

   // Fljande funktion implementerar djup kopiering och tilldelning.
   static Node_Pointer clone(const Node_Pointer);

   // Fljande hjlpfunktioner r interna fr AVL_Tree_Node.
   static void single_rotate_with_left_child(Node_Pointer&);
   static void single_rotate_with_right_child(Node_Pointer&);
   static void double_rotate_with_left_child(Node_Pointer&);
   static void double_rotate_with_right_child(Node_Pointer&);

   static void calculate_height(const Node_Pointer);
   static int  node_height(const Node_Pointer);

   static int max(int x, int y) { return (x > y ? x : y); }

   static void indent(std::ostream& os, int level);
};

/*
 * AVL-rotationer och tillhrande hjlpfunktioner.
 */

/**
 * Returnera en nod/subtrds hjd.
 */
template<typename Comparable>
int
AVL_Tree_Node<Comparable>::
node_height(const Node_Pointer p)
{
   return (p != 0 ? p->height_ : -1);
}

/**
 * Justera hjden fr noden p.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
calculate_height(const Node_Pointer p)
{
   p->height_ = 1 + max(node_height(p->left_), node_height(p->right_));
}

/**
 * enkelrotation vnster med vnster barn med noden k2 som pivot.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
single_rotate_with_left_child(Node_Pointer& k2)
{
   Node_Pointer k1 = k2->left_;

   k2->left_ = k1->right_;
   k1->right_ = k2;

   k2->height_ = max(node_height(k2->left_), node_height(k2->right_)) + 1;
   k1->height_ = max(node_height(k1->left_), k2->height_) + 1;

   k2 = k1;
}

/**
 * Enkelrotation med hger barn med noden k1 som pivot.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
single_rotate_with_right_child(Node_Pointer& k1)
{
   Node_Pointer k2 = k1->right_;

   k1->right_ = k2->left_;
   k2->left_ = k1;

   k1->height_ = max(node_height(k1->right_), node_height(k1->left_)) + 1;
   k2->height_ = max(node_height(k2->right_), k1->height_) + 1;

   k1 = k2;
}

/**
 * dubbelrotation vnster-hger med den noden k3 som pivot.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
double_rotate_with_left_child(Node_Pointer& k3)
{
   single_rotate_with_right_child(k3->left_);
   single_rotate_with_left_child(k3);
}

/**
 * Dubbelrotation hger-vnster med noden k3 som pivot.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
double_rotate_with_right_child(Node_Pointer& k3)
{
   single_rotate_with_left_child(k3->right_);
   single_rotate_with_right_child(k3);
}

/*
 * vriga medlemsfunktioner fr AVL_Tree_Node
 */

/**
 * Stt in x i trdet som ett nytt lv. Kontrollera sedan trdets balans och 
 * justera vid behov.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
insert(const Comparable& x, Node_Pointer& t)
{
   if (t == 0) 
   {
      t = new Node(x);
      return;
   }

   if (x < t->element_) 
   {
      insert(x, t->left_);

      if (node_height(t->left_) - node_height(t->right_) == 2)
         if (x < t->left_->element_)
            single_rotate_with_left_child(t);
         else
            double_rotate_with_left_child(t);
      else
         calculate_height(t); 
   }
   else if (t->element_ < x) 
   {
      insert(x, t->right_);

      if (node_height(t->right_) - node_height(t->left_) == 2)
         if (t->right_->element_ < x)
            single_rotate_with_right_child(t);
         else
            double_rotate_with_right_child(t);
      else
         calculate_height(t);
   }
   else 
   {
      throw avl_tree_error("insttning: finns redan");
   }
}

template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
remove(const Comparable& x, Node_Pointer& t)
{
   if (t == 0) 
   {
      return;  // Hr kan ett undantag genereras i stllet ...
   }

   if (x < t->element_) 
   {
      remove(x, t->left_);

      /* Is the right tree too big? */
      if (node_height(t->right_) - node_height(t->left_) == 2)
      {
         if (node_height(t->right_->left_) - node_height(t->right_->right_) > 0)
         {
            cout << "Performing double_rotate_with_right_child." << endl;
            double_rotate_with_right_child(t);
         }
         else
         {
            cout << "Performing single_rotate_with_right_child." << endl;
            single_rotate_with_right_child(t);
         }
      }
   } 
   else if (t->element_ < x) 
   {
      remove(x, t->right_);

      /* Is the left tree too big? */
      if (node_height(t->left_) - node_height(t->right_) == 2)
      {
         if (node_height(t->left_->left_) - node_height(t->left_->right_) < 0)
         {
            cout << "Peforming double_rotate_with_left_child" << endl;
            double_rotate_with_left_child(t);
         }
         else
         {
            cout << "Performing single_rotate_with_left_child" << endl;
            single_rotate_with_left_child(t);
         }
      }
   } 
   else 
   {
      // Skt vrde finns i noden t
      Node_Pointer tmp;

      if (t->left_ != 0 && t->right_ != 0) {
         // Noden har tv barn och erstts med inorder efterfljare
         tmp = find_min(t->right_);
         t->element_ = tmp->element_;
         remove(t->element_, t->right_);
      }
      else {
         // Noden har inget eller ett barn
         tmp = t;

         if (t->left_ == 0)
            t = t->right_;
         else
            t = t->left_;

         delete tmp;

         return; /* So we don't try to call calculate_height() for this node we just deleted. */
      }
   }

   /* Probably don't have to call this often but it makes for easier code. */
   calculate_height(t);
}

/**
 * Skriv ut elementen i stigande ordning.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
print(ostream& os, const Node_Pointer t)
{
   if (t != 0) 
   {
      print(os, t->left_);
      os << t->element_ << " ";
      print(os, t->right_);
   }
}

/**
 * print_tree() gr en trdutskrift.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
print_tree(ostream& os, const Node_Pointer t, int depth)
{
   if (t != 0) 
   {
      print_tree(os, t->right_, depth + 1); 

      if (t->right_ != 0) 
      {
         indent(os, depth);
         os << " /" << endl;
      }

      indent(os, depth);
      os << t->element_ << endl;

      if (t->left_ != 0) 
      {
         indent(os, depth);
         os << " \\" << endl;
      }

      print_tree(os, t->left_, depth + 1); 
   }
}

/**
 * Gr en kopia av trdet t.
 */
template<typename Comparable>
AVL_Tree_Node<Comparable>*
AVL_Tree_Node<Comparable>::
clone(const Node_Pointer t)
{
   if (t != 0)
      return new Node(t->element_, clone(t->left_), clone(t->right_));
   else
      return 0;
}

/**
 * Sk efter x i trdet. Om vrdet hittas returneras pekaren till den noden,
 * annars en tompekare.
 */
template<typename Comparable>
AVL_Tree_Node<Comparable>*
AVL_Tree_Node<Comparable>::
find(const Comparable& x, const Node_Pointer t)
{
   if (t == 0)
      return 0;
   else if (x < t->element_)
      return find(x, t->left_);
   else if (t->element_ < x)
      return find(x, t->right_);
   else
      return t;
}

/**
 * Sk det mista vrdet i trdet, dvs den vnstraste noden i trdet, dr det 
 * minsta vrdet finns och returnerar pekaren till den noden. Om trdet r 
 * tomt returneras en tompekare.
 */
template<typename Comparable>
AVL_Tree_Node<Comparable>*
AVL_Tree_Node<Comparable>::
find_min(const Node_Pointer t)
{
   if (t == 0)
      return 0;
   else if (t->left_ == 0)
      return t;
   else
      return find_min(t->left_);
}

/**
 * Sk det mista vrdet i trdet, dvs den hgraste noden i trdet, dr det 
 * strsta vrdet finns och returnerar pekaren till den noden. Om trdet r
 * tomt returneras en tompekare.
 */
template<typename Comparable>
AVL_Tree_Node<Comparable>*
AVL_Tree_Node<Comparable>::
find_max(const Node_Pointer t)
{
   Node_Pointer p = t;
   if (p != 0) 
   {
      while (p->right_ != 0)
         p = p->right_;
   }
   return p;
}

/**
 * Ta bort alla noder i trdet t.
 */
template<typename Comparable>
void
AVL_Tree_Node<Comparable>::
clear(Node_Pointer& t)
{
   if (t != 0) 
   {
      clear(t->left_);
      clear(t->right_);
      delete t;
      t = 0;
   }
}

/**
 * indent() gr indrag motsvarande det aktuella trddjupet.
 */
template<typename Comparable>
void 
AVL_Tree_Node<Comparable>::
indent(ostream& os, int level)
{
   for (int i = 0; i < level; ++i)
      os << "  ";
}

/*
 * Konstruktorer, destruktor och tilldelning fr AVL_Tree 
 */

/**
 * Standardkonstruktor fr AVL-trdet. Stter rotpekare och pekaren till
 * aktuell (senast beskta) nod till tompekare.
 */
template<typename Comparable>
AVL_Tree<Comparable>::
AVL_Tree() : root_(0), current_(0)
{
}

/**
 * Kopieringskonstruktor fr AVL-trdet. Gr en djup kopiering av det trd
 * som ges som initialvrde.
 */
template<typename Comparable>
AVL_Tree<Comparable>::
AVL_Tree(const AVL_Tree& value) : root_(Node::clone(value.root_)), current_(0)
{
}

/**
 * Destruktor fr AVL-trdet. terlmnar alla noder i trdet.
 */
template<typename Comparable>
AVL_Tree<Comparable>::
~AVL_Tree()
{
   Node::clear(root_);
}

/**
 * Tilldelningsoperator. Gr en djup kopiering av det trd som ges som 
 * hgerargument.
 */
template<typename Comparable>
AVL_Tree<Comparable>&
AVL_Tree<Comparable>::
operator=(AVL_Tree<Comparable> rhs)
{
   swap(rhs);
   return *this;
}

/*
 * Publika medlemsfunktioner fr AVL_Tree
 */

/**
 * Stt in x i trdet.
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
insert(const Comparable& x)
{
   Node::insert(x, root_);
}

/**
 * Ta bort x ur trdet.
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
remove(const Comparable& x)
{
   //throw avl_tree_error("remove: ska implementeras!");
   Node::remove(x, root_);
}

/**
 * Returnera sant om x finns i trdet, annars falskt.
 */
template<typename Comparable>
bool
AVL_Tree<Comparable>::
member(const Comparable& x) const
{
   current_ = Node::find(x, root_);
   return current_ != 0;
}

/**
 * Returnera sant om x finns i trdet, annars falskt.
 */
template<typename Comparable>
Comparable&
AVL_Tree<Comparable>::
retrieve()
{
   if (current_ == 0)
      throw avl_tree_error("tkomst: inget aktuellt element");
   return current_->element;;
}

/**
 * Sk upp vrdet x i trdet. Returnera referens till x.
 */
template<typename Comparable>
Comparable&
AVL_Tree<Comparable>::
find(const Comparable& x) const
{
   current_ = Node::find(x, root_);

   if (current_ == 0)
      throw avl_tree_error("skt vrde finns ej i trdet");

   return current_->element_;
}

/**
 * Sk upp det minsta vrdet i trdet. Returnera referens till vrdet.
 */
template<typename Comparable>
Comparable&
AVL_Tree<Comparable>::
find_min() const
{
   if (root_ == 0)
      throw avl_tree_error("frsk att finna minst i tomt trd");

   current_ = Node::find_min(root_);
   return current_->element_;
}

/**
 * Sk upp det strsta vrdet i trdet. Returnera referens till vrdet.
 */
template<typename Comparable>
Comparable&
AVL_Tree<Comparable>::
find_max() const
{
   if (root_ == 0)
      throw avl_tree_error("frsk att finna strst i tomt trd");

   current_ = Node::find_max(root_);
   return current_->element_;
}

/**
 * Returnerar sant om trdet r tomt, falskt annars.
 */
template<typename Comparable>
bool
AVL_Tree<Comparable>::
empty() const
{
   return root_ == 0;
}

/**
 * Tm trdet.
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
clear()
{
   Node::clear(root_);
}

/**
 * Skriv ut alla element i trdet i stigande ordning.anropar AVL_Tree_Node::print_tree()
 * fr att gra sjlva utskriften.
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
print(ostream& os) const
{
   Node::print(os, root_);
}

/**
 * print_tree() anropar AVL_Tree_Node::print_tree() fr att gra en 
 * trdutskrift.
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
print_tree(ostream& os) const
{
   Node::print_tree(os, root_, 0);
}

/**
 * Byt innehll p this och other. (Borde current_ sttas till 0 hellre?)
 */
template<typename Comparable>
void
AVL_Tree<Comparable>::
swap(AVL_Tree<Comparable>& other)
{
   std::swap(root_, other.root_);
   std::swap(current_, other.current_);
}

/**
 * Byt innehll p x och y.
 */
template<typename Comparable>
void swap(AVL_Tree<Comparable>& x, AVL_Tree<Comparable>& y)
{
   x.swap(y);
}

/**
 * Gr en trdutskrift.
 */
template<typename Comparable>
ostream& operator<<(ostream& os, const AVL_Tree<Comparable>& tree)
{
   tree.print_tree(os);
   return os;
}
